//----------------------------------------------------------------------
// App.jsx
// Author: Bob Dondero
//----------------------------------------------------------------------

import * as React from 'react';
const VITE_SERVER_URL = import.meta.env.VITE_SERVER_URL

//----------------------------------------------------------------------

function PennyHeader() {
   const [datetime, setDatetime] = React.useState(new Date());

   function updateHeader() {
      window.setInterval(
         () => {setDatetime(new Date());},
         1000
      );
   }
   React.useEffect(updateHeader, []);

   let hours = datetime.getHours();
   let ampm = (hours < 12) ? 'morning' : 'afternoon';
   return (
      <div>
         <hr />
         Good {ampm} and welcome to Penny.com
         <hr />
      </div>
   );
}

//----------------------------------------------------------------------

function PennySearch() {
   const [author, setAuthor] = React.useState('');
   const [books, setBooks] = React.useState([]);
   function fetchBooks() {
      function handleResponse() {
         if (this.status !== 200) {
            alert('Error: Failed to fetch data from server');
            return;
         }
         let books = JSON.parse(this.response);
         setBooks(books);
      }
      function handleError() {
         alert('Error: Failed to fetch data from server');
      }
      let encodedAuthor = encodeURIComponent(author);
      let url = VITE_SERVER_URL +
         '/api/searchresults?author=' + encodedAuthor;
      let request = new XMLHttpRequest();
      request.onload = handleResponse;
      request.onerror = handleError;
      request.open('GET', url);
      request.send();
      return () => {request.abort();}
   }
   React.useEffect(fetchBooks, [author]);

   let timer = null;
   function debouncedSetAuthor(author) {
      clearTimeout(timer);
      timer = setTimeout(()=>{setAuthor(author);}, 500);
   }

   return (
      <div>
         <h1>Author Search</h1>
         Please enter an author name:&nbsp;
         <input
             type="text"
             onInput={ (event) => {
                debouncedSetAuthor(event.target.value);
             } }
             autoFocus
         />
         <hr />
         {books.map((book) => (
            <div key={book.isbn}>
               {book.isbn}:&nbsp;
               <strong>{book.author}</strong>:&nbsp;
               {book.title}
               <br />
            </div>
         ))}
      </div>
   );
}

//----------------------------------------------------------------------

function PennyFooter() {
   const [datetime, setDatetime] = React.useState(new Date());

   function updateFooter() {
      window.setInterval(
         () => {setDatetime(new Date());},
         1000
      );
   }
   React.useEffect(updateFooter, []);

   return (
      <div>
         <hr />
         Date and time: {datetime.toLocaleString()}
         <br />
         Created by&nbsp;
         <a href="https://www.cs.princeton.edu/~rdondero">
         Bob Dondero</a>
      </div>
   );
}

//----------------------------------------------------------------------

function App() {
   return (
      <div>
         <PennyHeader />
         <PennySearch />
         <PennyFooter />
      </div>
   );
}

//----------------------------------------------------------------------
export default App;
