//----------------------------------------------------------------------
// App.jsx
// Author: Bob Dondero
//----------------------------------------------------------------------

import * as React from 'react';

const VITE_SERVER_URL = import.meta.env.VITE_SERVER_URL

//----------------------------------------------------------------------

function PennyHeader() {
   const [datetime, setDatetime] = React.useState(new Date());

   function updateHeader() {
      window.setInterval(
         () => {setDatetime(new Date());},
         1000
      );
   }

   React.useEffect(updateHeader, []);

   let hours = datetime.getHours();
   let ampm = (hours < 12) ? 'morning' : 'afternoon';

   return (
      <div>
         <hr />
         Good {ampm} and welcome to Penny.com
         <hr />
      </div>
   );
}

//----------------------------------------------------------------------

function PennyInput(props) {
   return (
      <div>
         <h1>Author Search</h1>
         Please enter an author name:&nbsp;
         <input
             type="text"
             onInput={ (event) => {
                props.callback(event.target.value);
             } }
             autoFocus
         />
         <hr />
      </div>
   );
}

//----------------------------------------------------------------------

function PennyOutput(props) {
   return (
      <div>
         {props.books.map((book) => (
            <div key={book.isbn}>
               {book.isbn}:&nbsp;
               <strong>{book.author}</strong>:&nbsp;
               {book.title}
               <br />
            </div>
         ))}
      </div>
   );
}

//----------------------------------------------------------------------

function PennySearch() {
   const [author, setAuthor] = React.useState('');
   const [books, setBooks] = React.useState([]);

   function fetchBooks() {
      let encodedAuthor = encodeURIComponent(author);
      let url = VITE_SERVER_URL +
         '/api/searchresults?author=' + encodedAuthor;
      let request = new XMLHttpRequest();
      request.onload = () => {
         if (request.status === 200)
            setBooks(JSON.parse(request.response));
         else
            alert('Error: Failed to fetch data from server');
      };
      request.onerror = () => {
         alert('Error: Failed to fetch data from server');
      };
      request.open('GET', url);
      request.send();
      return () => {request.abort();}
   }

   //function fetchBooks() {
   //   let controller = new AbortController();
   //   let encodedAuthor = encodeURIComponent(author);
   //   let url = VITE_SERVER_URL +
   //    '/api/searchresults?author=' + encodedAuthor;
   //   fetch(url, {signal: controller.signal})
   //      .then((resp) => {
   //         if (! resp.ok) throw new Error();
   //         return resp.text();
   //      })
   //      .then((text) => {setBooks(JSON.parse(text));})
   //      .catch(() => {
   //         alert('Error: Failed to fetch data from server');
   //      });
   //   return () => {controller.abort();};
   //}

   //function fetchBooks() {
   //   let controller = new AbortController();
   //   let encodedAuthor = encodeURIComponent(author);
   //   let url = VITE_SERVER_URL +
   //    '/api/searchresults?author=' + encodedAuthor;
   //   async function fetchBooksHelp() {
   //      try {
   //         let resp =
   //            await fetch(url, {signal: controller.signal});
   //         if (! resp.ok) throw new Error();
   //         let text = await resp.text();
   //         setBooks(JSON.parse(text));
   //      }
   //      catch (err) {
   //         alert('Error: Failed to fetch data from server');
   //      }
   //   }
   //   fetchBooksHelp();
   //   return () => {controller.abort();};
   //}

   React.useEffect(fetchBooks, [author]);

   let timer = null;

   function debouncedSetAuthor(author) {
      clearTimeout(timer);
      timer = setTimeout(()=>{setAuthor(author);}, 500);
   }

   return (
      <div>
          <PennyInput callback={debouncedSetAuthor} />
          <PennyOutput books={books} />
      </div>
   );
}

//----------------------------------------------------------------------

function PennyFooter() {
   const [datetime, setDatetime] = React.useState(new Date());

   function updateFooter() {
      window.setInterval(
         () => {setDatetime(new Date());},
         1000
      );
   }

   React.useEffect(updateFooter, []);

   return (
      <div>
         <hr />
         Date and time: {datetime.toLocaleString()}
         <br />
         Created by&nbsp;
         <a href="https://www.cs.princeton.edu/~rdondero">
         Bob Dondero</a>
      </div>
   );
}

//----------------------------------------------------------------------

function App() {
   return (
      <div>
         <PennyHeader />
         <PennySearch />
         <PennyFooter />
      </div>
   );
}

//----------------------------------------------------------------------

export default App;
